﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Snake
{
    public partial class MainWindow : Window
    {
        const int width = 30;
        const int height = 20;
        const int margin = 5;
        const int gridSize = 10; // Gitterabstand

        Random random = new Random();
        System.Windows.Threading.DispatcherTimer timer = new System.Windows.Threading.DispatcherTimer();

        int[] xSnake; // eleganter: List<Tuple<int, int>>
        int[] ySnake;
        int xFood;
        int yFood;
        int direction = -1; // 0 1 2 3 = rechts rauf links runter; -1 = noch nicht gestartet

        public MainWindow()
        {
            InitializeComponent();

            reset();

            timer.Interval = TimeSpan.FromSeconds(0.1);
            timer.Tick += animate;
            timer.IsEnabled = true;
        }

        void reset()
        {
            xSnake = new int[]{width / 2};
            ySnake = new int[]{height / 2};

            placeFood();
            draw();
        }

        private void Window_KeyDown(object sender, KeyEventArgs e)
        {
            int oldDirection = direction;

            switch (e.Key)
            {
                case Key.Right:
                    direction = 0;
                    break;
                case Key.Up:
                    direction = 1;
                    break;
                case Key.Left:
                    direction = 2;
                    break;
                case Key.Down:
                    direction = 3;
                    break;
                case Key.C: // nur zum Test der reset-Methode
                    reset();
                    return;
                default:
                    break;
            }

            // die Schlange nicht entlang sich selbst zurückbewegen
            bool accept = true;
            if(xSnake.Length > 1)
            {
                int xNew = xSnake[0];
                int yNew = ySnake[0];
                switch (direction)
                {
                    case 0:
                        xNew++;
                        break;
                    case 1:
                        yNew--;
                        break;
                    case 2:
                        xNew--;
                        break;
                    case 3:
                        yNew++;
                        break;
                }
                if(xNew == xSnake[1] && yNew == ySnake[1])
                {
                    accept = false;
                }
            }
            if(!accept)
            {
                direction = oldDirection;
            }
        }

        void animate(object sender, EventArgs e)
        {
            if(direction == -1)
            {
                return;
            }

            // Wo soll der Kopf hin?
            int xNew = xSnake[0];
            int yNew = ySnake[0];
            switch (direction)
            {
                case 0:
                    xNew++;
                    break;
                case 1:
                    yNew--;
                    break;
                case 2:
                    xNew--;
                    break;
                case 3:
                    yNew++;
                    break;
            }

            if (xNew == xFood && yNew == yFood) // Treffer
            {
                // Schlange vorne um Position des Essens verlängern

                int[] xLongerSnake = new int[xSnake.Length + 1];
                int[] yLongerSnake = new int[xSnake.Length + 1];
                xLongerSnake[0] = xNew;
                yLongerSnake[0] = yNew;
                for (int i = 0; i < xSnake.Length; i++)
                {
                    xLongerSnake[i + 1] = xSnake[i];
                    yLongerSnake[i + 1] = ySnake[i];
                }
                xSnake = xLongerSnake;
                ySnake = yLongerSnake;
                placeFood();
            }
            else
            {
                // alle eins weiterrutschen

                for (int i = xSnake.Length - 1; i > 0; i--)
                {
                    xSnake[i] = xSnake[i - 1];
                    ySnake[i] = ySnake[i - 1];
                }
                xSnake[0] = xNew;
                ySnake[0] = yNew;
            }

            draw();
        }

        void draw()
        {
            myCanvas.Children.Clear();

            for (int i = 0; i < xSnake.Length; i++)
            {
                Ellipse ellipse = new Ellipse();
                ellipse.Width = gridSize;
                ellipse.Height = gridSize;
                ellipse.Fill = Brushes.Green;
                myCanvas.Children.Add(ellipse);
                Canvas.SetLeft(ellipse, (xSnake[i] + margin) * gridSize);
                Canvas.SetTop(ellipse, (ySnake[i] + margin) * gridSize);
            }

            Ellipse ellipse2 = new Ellipse();
            ellipse2.Width = gridSize;
            ellipse2.Height = gridSize;
            ellipse2.Fill = Brushes.Red;
            myCanvas.Children.Add(ellipse2);
            Canvas.SetLeft(ellipse2, (xFood + margin) * gridSize);
            Canvas.SetTop(ellipse2, (yFood + margin) * gridSize);
        }

        void placeFood()
        {
            int numFreeSquares = width * height - xSnake.Length;
            if (numFreeSquares == 0)
            {
                MessageBox.Show("Glückwunsch!");
                reset();
            }
            else
            {
                // Welche Felder sind schon von der Schlange belegt?
                bool[] squareOccupied = new bool[width * height];
                for (int i = 0; i < xSnake.Length; i++)
                {
                    if (xSnake[i] >= 0 && xSnake[i] < width && ySnake[i] >= 0 && ySnake[i] < height)
                    {
                        squareOccupied[width * ySnake[i] + xSnake[i]] = true;
                    }
                }

                int mouseSquare = random.Next(numFreeSquares);
                // Gehe zum ersten freien Platz.
                int k = 0;
                while (squareOccupied[k])
                {
                    k++;
                }
                // Zähle nun bis zum freien Platz mit der Nummer mouseSquare und nehme den.
                for (int j = 0; j < mouseSquare; j++)
			    {
                    // Finde den nächsten freien Platz.
                    k++;
                    while (squareOccupied[k])
                    {
                        k++;
                    }			 
			    }
			    xFood = k % width;
                yFood = k / width;
            }
        }
    }
}
